# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega. 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#
import ldap
from user import LdapControl

class LdapGroup(object):
    """
    this class represents an ldap group.
    in ou=Groups,dc=satega,dc=local
    
    the users are saved and accassed through
    self.members. In this list there are the FQDN (dn)
    """
    def __init__(self):
        """
        If you want to load a group
        use the LdapGroupControl
        
        
        """
        pass
        
    
    def __eq__(self,b):
        """
        allow group objects to be comparted to each other
        and to strings
        """
        if isinstance(b,LdapGroup):
            if self.name == b.getGroupName():
                return True
        if isinstance(b,basestring):
            if self.name == b:
                return True
        return False
    
    def __str__(self):
        return "LdapGroup %s" % (self.name,)
        
    def __repr__(self):
        return self.__str__()
        
    def __contains__(self, item):
        """
        overrides the system contain method
        to allow the following code:
        >>> member = LdapControl.makeDN('root') in groups
        """
        if item in self.members:
            return True
        else:
            return False
    
    def getGroupName(self):
        """
        @return the groupName example: admins (not in DN syntax)
        """
        return self.name
    
    def getMembersasObjects(self):
        """
        instead of returning just the dn string
        this return LdapUser objects
        @return List of ldapUser objects
        """
        objects = []
        if len(self.members) == 0:
            return []
        for dn in self.members:
            uid = LdapControl.UIDfromDN(dn)
            user = LdapControl.loadUser(uid)
            if user is None:
                continue
            objects.append( user )
            
        return objects
    
    def addMember(self,dn):
        """
        adds the member to the group
        @param dn = FQN name uid=root,ou=User,dc=satega,dc=local
        
        Example:
        >>> group = LdapGroupControl.loadGroup('mygroup')
        >>> group.addMember(LdapControl.makeDN('newuser'))
        >>> group.save()
        """
        self.members.append(dn)
        
    def removeMember(self,dn):
        self.members.remove(dn)
    
    def save(self):
        """
        saves the changed members to the ldap system
        @return true or exception
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
         #ldap does not understand unicode
        for i in range(len(self.members)):
            self.members[i] = self.members[i].encode()
        
        
        if len(self.members) == 0:
            self.members = ""
        modlist = (
                   (ldap.MOD_REPLACE , "uniqueMember", self.members),
                      )
        
        ldapConnection.modify_s("cn=%s,ou=Groups,dc=satega,dc=local" % (self.name,) , modlist)
        ldapConnection.unbind_s()
        return True
    
class LdapGroupControl(object):
    
    @staticmethod
    def loadGroup(groupName):
        group = LdapGroup()
        group.members = []
        group.name = groupName
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        searchStr = "cn=%s,ou=Groups,dc=satega,dc=local" % (groupName,)
        res = ldapConnection.search_s( searchStr, ldap.SCOPE_BASE, attrlist=("uniqueMember",))
        for user in res[0][1]["uniqueMember"]:
            if user != "":
                group.members.append(user)
        ldapConnection.unbind_s()
        return group
    
    @staticmethod
    def removeGroup(groupname):
        """
        removes a group from the system
        @param groupname the name in cn syntax to remove
        """
        if groupname == 'admins':
            raise Exception('can not delete admin group')
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        ldapConnection.delete_s("cn=%s,ou=Groups,dc=satega,dc=local" % (groupname,))
        ldapConnection.unbind_s()
        return True
    
    @staticmethod
    def createGroup(groupname):
        """
        creates a new group on the ldap server. To add members to the
        group you have to load the group right after creating it.
        Example:
        >>> LdapGroupControl.createGroup('mynewone')
        >>> group = LdapGroupControl.loadGroup('mynewone')
        >>> group.addMember(LdapControl.makeDN('frank'))
        >>> group.save()
        
        @param groupname the name for the new group.
        @return True if the group was created
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        modlist =  (
                    ("objectClass", ("groupOfUniqueNames",) ),
                    ("cn", (groupname.encode(),) ),
                    ("uniqueMember",""),
                   )
        ldapConnection.add_s("cn=%s,ou=Groups,dc=satega,dc=local" % (groupname,) , modlist)
        ldapConnection.unbind_s()
        return True
    
    @staticmethod
    def listGroups():
        """
        @return all the groups in ou=Groups,dc=satega,dc=local (list with 'cn')
        
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        res = ldapConnection.search_s("ou=Groups,dc=satega,dc=local", ldap.SCOPE_ONELEVEL, attrlist=("*",))
        groups = []
        for group in res:
            if "groupOfUniqueNames" not in group[1]['objectClass']:
                #only groupOfUnique names is supported
                continue
            if "cn" in group[1]:
                cn = group[1]['cn'][0]
                group = LdapGroupControl.loadGroup(cn)
                groups.append(group)
            else:
                #nothing we ignore users without uid
                continue
        ldapConnection.unbind_s()
        return groups
    
