# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
# Svein Hansen <sveinh@gmail.com>
#
# This file is part of Satega (www.satega.org)
#
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
#



from satega.lib.shellProcess import ShellProcess,shLaunch
from ConfigParser import ConfigParser

import time
import os.path
from pylons import config

# Module Error constants

INVALID_PARAMETER = 20000
SERVER_NOT_INSTALLED = 20001

import logging
log = logging.getLogger(__name__)

class MusicServer(object):
    """Class to handle MusicServer configuration 

    @note: It seems that if mt-daap is re-installed, the previous mp3_path is cached
           and services the previous music files, even with the default (non-valid)
           mp3_path (/home/media/music) in the default install. The solution is to
           trigger a "full scan" to reset the information. This can be implemented later.
           
           The following command will trigger a full scan:
           
           wget http://none:<masterpw>@localhost:3689/xml-rpc?method=post&method=rescan&full=1

           Better:
           wget --http-user=admin --http-password=mt-daapd http://127.0.0.1:3689/xml-rpc?method=rescan&full=1 --read-timeout=1
    """


    def __init__(self):
        """ Initialise the class
    
        """
        self.config_file = ''
        self.init_script = ''
        self.initialize()


    def initialize(self):
        """Check that the module has required configurations
        
        @return: True if all requirements are met
        """
        #self.config_file = self.module_config.get('main', 'config_file')
        #self.init_script = self.module_config.get('main', 'init_script')
        self.config_file = config['musicserver.config_file']
        self.init_script = config['musicserver.init_script']
        self.enabled = config['musicserver.enabled']
        
        if not os.path.exists(self.config_file):
            log.error("Music Server configuration file does not exist. " + 
                              "Is mt-daapd installed? " + 
                              "config file:" + self.config_file)
            return False

        if not os.path.exists(self.init_script):
            log.error("Music Server init.d script does not exist. " + 
                              "Is mt-daapd installed? " + 
                              "init.d script:" + self.init_script)
            return False
            
        # enable auto start on boot and start the service process
        if self.enabled:
            ShellProcess("update-rc.d -f mt-daapd multiuser").execute()
            ShellProcess("%s start" % self.init_script).execute()

        return True
    
    def getServerName(self):
        """
        @returns the servername displayed to the music streaming
        clients
        """
        return self.getConfiguration()['servername']
        
    def getmp3dirs(self):
        return self.getConfiguration()['mp3_dirs']

#===============================================================================
#    Private methods
#===============================================================================

    def __readConfigurationFile(self):
        """Return the mt-daapd configuration file as a ConfigParser object
        
        @return: ConfigParser
        """

        # Read the configuration file for mt-daapd
        mt_daapd_Config = ConfigParser()

        try:
            mt_daapd_Config.readfp(open(self.config_file))
        except:
            raise Exception(0,
                              "Error reading mt-daapd.conf file")

        if not mt_daapd_Config.sections():
            raise Exception(0,
                                "mt-daapd.conf file does not contain valid values")
            
        return mt_daapd_Config
        
        
#===============================================================================
#    Public methods
#===============================================================================

    
    def getConfiguration(self):
        """Return the required values from the mt-daapd configuration file
        
        @return: Dictionary with configuration values:
                    mp3_dirs: Comma separated string with path names
                    servername: String with name of the server (announced to the clients)
        """

        # Read the configuration file from disk
        mt_daapd_config = self.__readConfigurationFile()
        
        # Create the dictionary with values from the configuration file
        musicconfig = {}

        musicconfig ['mp3_dirs'] = mt_daapd_config.get('general', 'mp3_dir')
        musicconfig ['servername'] = mt_daapd_config.get('general', 'servername')
        
        return musicconfig 

    
    def saveConfiguration(self, mp3_dir, servername):
        """Update the selected values in the mt-daapd configuration file
        
        @param mp3_dir: Comma separated string with path names
        @param servername: String with name of the server (announced to the clients)
        @return: Dictionary with newly set configuration values (same as input values):
        """
        
        # Check that the mp3_dir is set and not empty
        if(not mp3_dir or (mp3_dir is '')):
            raise SategaFault(INVALID_PARAMETER,
                              "Music folder value cannot be empty")
         
        # Check that the mp3_dir is set and not empty
        if(not servername or (servername is '')):
            raise SategaFault(INVALID_PARAMETER,
                              "Server name value cannot be empty")
             
        # Check that the mp3_dir values are valid existing folders
        folders = mp3_dir.split(',')
        
        for folder in folders:
            folder = folder.strip()
            if not os.path.isdir(folder):
                raise Exception(INVALID_PARAMETER,
                              "Music folder not found: %s" % folder)

        # Read the current configuration file from disk
        mt_daapd_config = self.__readConfigurationFile()
        
        # Update the new values
        mt_daapd_config.set('general', 'mp3_dir', mp3_dir)
        mt_daapd_config.set('general', 'servername', servername)

        # Write the configuration file to disk
        fp = file(self.config_file, "w")
        
        try:
            mt_daapd_config.write(fp)
        except:
            raise Exception(support.errorcodes.TECHNICAL_SERVER_ERROR,
                              "Error writing new configuration")
        finally:
            fp.close()
 
        log.debug("new mt-daapd configuration written")

        try:
            # Restart the mt-daap server process
            sp = ShellProcess ("%s restart" % self.init_script)
            sp.execute()
            time.sleep(1) # wait 1sec to let the server start
            sp = ShellProcess("wget  --http-user=admin --http-password=mt-daapd http://127.0.0.1:3689/xml-rpc?method=rescan&full=1 &> /dev/null")
            sp.execute()
            log.debug("mt-daapd daemon restarted")
        except:
            raise Exception(0,
                              "Error restarting Music Server process")

        # Re-read the new configuration and return stored values
        return self.getConfiguration()


    def disable_module(self):
        #SategaModule.disable_module(self)
        # Try to stop the service regardless if it's running or not
        ShellProcess("%s stop" % self.init_script).execute()
        # Disable auto start of service on boot
        ShellProcess("update-rc.d -f mt-daapd remove").execute()
        
        return True
        
